@@ -19,17 +19,9 @@
  * (C) Copyright 2005 Ethium, Inc.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
-#include <libnautilus-extension/nautilus-extension-types.h>
-#include <libnautilus-extension/nautilus-column-provider.h>
-#include <libnautilus-extension/nautilus-extension-types.h>
-#include <libnautilus-extension/nautilus-file-info.h>
-#include <libnautilus-extension/nautilus-info-provider.h>
-#include <libnautilus-extension/nautilus-menu-provider.h>
-#include <libnautilus-extension/nautilus-property-page-provider.h>
+#include <nautilus-extension.h>
 
 #include "nautilus-share.h"
 
@@ -68,12 +60,12 @@ typedef struct {
   GtkWidget *main; /* Widget that holds all the rest.  Its "PropertyPage" GObject-data points to this PropertyPage structure */
   
   GtkWidget *checkbutton_share_folder;
-  GtkWidget *hbox_share_name;
-  GtkWidget *hbox_share_comment;
   GtkWidget *entry_share_name;
   GtkWidget *checkbutton_share_rw_ro;
   GtkWidget *checkbutton_share_guest_ok;
   GtkWidget *entry_share_comment;
+  GtkWidget *label_share_name;
+  GtkWidget *label_share_comment;
   GtkWidget *label_status;
   GtkWidget *button_cancel;
   GtkWidget *button_apply;
@@ -85,16 +77,82 @@ typedef struct {
   gboolean is_dirty;
 } PropertyPage;
 
+typedef struct {
+  mode_t new_mode;
+  mode_t need_mask;
+  PropertyPage *page;
+} SharingDialogData;
+
 static void property_page_set_warning (PropertyPage *page);
 static void property_page_set_error (PropertyPage *page, const char *message);
 static void property_page_set_normal (PropertyPage *page);
+static void error_when_changing_permissions (GtkWidget *widget, const char *path);
+static void save_changed_permissions (const char *path, mode_t need_mask);
+static void property_page_check_sensitivity (PropertyPage *page);
+static void restore_saved_permissions (const char *path);
+
+static const char*
+entry_get_text (GtkEntry *entry)
+{
+#ifdef USE_NAUTILUS_3
+  return gtk_entry_get_text (entry);
+#endif
+#ifdef USE_NAUTILUS_4
+  return gtk_editable_get_text (GTK_EDITABLE (entry));
+#endif
+}
+
+static void
+entry_set_text (GtkEntry *entry, const gchar *text)
+{
+#ifdef USE_NAUTILUS_3
+  return gtk_entry_set_text (entry, text);
+#endif
+#ifdef USE_NAUTILUS_4
+  return gtk_editable_set_text (GTK_EDITABLE (entry), text);
+#endif
+}
+
+static void
+window_destroy (GtkWidget *widget)
+{
+#ifdef USE_NAUTILUS_3
+  gtk_widget_destroy (widget);
+#endif
+#ifdef USE_NAUTILUS_4
+  gtk_window_destroy (GTK_WINDOW (widget));
+#endif
+}
+
+static void
+check_button_set_active (GtkWidget *widget,
+                         gboolean   active)
+{
+#ifdef USE_NAUTILUS_3
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), active);
+#endif
+#ifdef USE_NAUTILUS_4
+  gtk_check_button_set_active (GTK_CHECK_BUTTON (widget), active);
+#endif
+}
+
+static gboolean
+check_button_get_active (GtkWidget *widget)
+{
+#ifdef USE_NAUTILUS_3
+  return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+#endif
+#ifdef USE_NAUTILUS_4
+  return gtk_check_button_get_active (GTK_CHECK_BUTTON (widget));
+#endif
+}
 
 static void
 property_page_validate_fields (PropertyPage *page)
 {
   const char *name;
 
-  name = gtk_entry_get_text (GTK_ENTRY (page->entry_share_name));
+  name = entry_get_text (GTK_ENTRY (page->entry_share_name));
   
   if (g_utf8_strlen (name, -1) <= 12)
     property_page_set_normal (page);
@@ -102,19 +160,110 @@ property_page_validate_fields (PropertyPage *page)
     property_page_set_warning (page);
 }
 
+static ShareInfo
+get_share_info (PropertyPage *page)
+{
+  ShareInfo share_info;
+
+  share_info.path = page->path;
+  share_info.share_name = (char *) entry_get_text (GTK_ENTRY (page->entry_share_name));
+  share_info.comment = (char *) entry_get_text (GTK_ENTRY (page->entry_share_comment));
+  share_info.is_writable = check_button_get_active (page->checkbutton_share_rw_ro);
+  share_info.guest_ok = check_button_get_active (page->checkbutton_share_guest_ok);
+
+  return share_info;
+}
+
 static gboolean
-message_confirm_missing_permissions (GtkWidget *widget, const char *path, mode_t need_mask)
+property_page_apply_changes (PropertyPage *page,
+                             gboolean      modified)
+{
+  GError *error = NULL;
+  ShareInfo share_info;
+  gboolean retval;
+  gboolean is_shared = check_button_get_active (page->checkbutton_share_folder);
+
+  share_info = get_share_info (page);
+
+  retval = shares_modify_share (share_info.path, is_shared ? &share_info : NULL, &error);
+
+  if (!retval)
+    {
+      property_page_set_error (page, error->message);
+      g_error_free (error);
+
+      /* Since the operation failed, we restore things to the way they were */
+      if (modified)
+        restore_saved_permissions (page->path);
+    }
+  else
+    {
+      property_page_validate_fields (page);
+      nautilus_file_info_invalidate_extension_info (page->fileinfo);
+    }
+
+  if (!is_shared)
+    restore_saved_permissions (page->path);
+
+  /* update initially shared state, so that we may undo later on */
+  if (retval)
+    {
+      page->was_initially_shared = is_shared;
+      page->is_dirty = FALSE;
+    }
+
+  return retval;
+}
+
+static void
+on_missing_permissions_dialog_response (GtkDialog *dialog,
+                                        int        resp_code,
+                                        gpointer   user_data)
+{
+  SharingDialogData *data = user_data;
+  gboolean samba_ret_val = FALSE;
+
+  if (resp_code == GTK_RESPONSE_ACCEPT)
+    {
+      if (chmod (data->page->path, data->new_mode) != 0)
+        {
+          error_when_changing_permissions (data->page->main, data->page->path);
+        }
+
+      save_changed_permissions (data->page->path, data->need_mask);
+      samba_ret_val = property_page_apply_changes (data->page, TRUE);
+    }
+
+  if (samba_ret_val)
+    {
+      if (data->page->standalone_window)
+        window_destroy (data->page->standalone_window);
+      else
+        property_page_check_sensitivity (data->page);
+    }
+
+  window_destroy (GTK_WIDGET (dialog));
+  g_free (data);
+}
+
+static void
+message_confirm_missing_permissions (SharingDialogData *data)
 {
   GtkWidget *toplevel;
   GtkWidget *dialog;
   char *display_name;
-  gboolean result;
-
-  toplevel = gtk_widget_get_toplevel (widget);
+  PropertyPage *page = data->page;
+  mode_t need_mask = data->need_mask;
+#ifdef USE_NAUTILUS_3
+  toplevel = gtk_widget_get_toplevel (page->main);
+#endif
+#ifdef USE_NAUTILUS_4
+  toplevel = GTK_WIDGET (gtk_widget_get_root (page->main));
+#endif
   if (!GTK_IS_WINDOW (toplevel))
     toplevel = NULL;
 
-  display_name = g_filename_display_basename (path);
+  display_name = g_filename_display_basename (page->path);
 
   dialog = gtk_message_dialog_new (toplevel ? GTK_WINDOW (toplevel) : NULL,
 				   0,
@@ -137,14 +286,19 @@ message_confirm_missing_permissions (GtkWidget *widget, const char *path, mode_t
 					    (need_mask & (S_IXGRP | S_IXOTH)) ? _("  - execute permission by others\n") : "");
   g_free (display_name);
 
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("Cancel"), GTK_RESPONSE_CANCEL);
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("Add the permissions automatically"), GTK_RESPONSE_ACCEPT);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
-  result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT;
-  gtk_widget_destroy (dialog);
+  gtk_widget_show (dialog);
+  g_signal_connect (dialog, "response", G_CALLBACK (on_missing_permissions_dialog_response), data);
+}
 
-  return result;
+static void
+on_error_dialog_response (GtkDialog *dialog,
+                          int response_id)
+{
+  window_destroy (GTK_WIDGET (dialog));
 }
 
 static void
@@ -154,7 +308,12 @@ error_when_changing_permissions (GtkWidget *widget, const char *path)
   GtkWidget *dialog;
   char *display_name;
 
+#ifdef USE_NAUTILUS_3
   toplevel = gtk_widget_get_toplevel (widget);
+#endif
+#ifdef USE_NAUTILUS_4
+  toplevel = GTK_WIDGET (gtk_widget_get_root (widget));
+#endif
   if (!GTK_IS_WINDOW (toplevel))
     toplevel = NULL;
 
@@ -168,8 +327,8 @@ error_when_changing_permissions (GtkWidget *widget, const char *path)
 				   display_name);
   g_free (display_name);
 
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
+  g_signal_connect (dialog, "response", G_CALLBACK (on_error_dialog_response), NULL);
+  gtk_widget_show (dialog);
 }
 
 static char *
@@ -310,21 +469,23 @@ restore_write_permissions (const char *path)
 }
 
 typedef enum {
-  CONFIRM_CANCEL_OR_ERROR,
+  CONFIRM_PENDING_CONFIRMATION,
   CONFIRM_NO_MODIFICATIONS,
   CONFIRM_MODIFIED
 } ConfirmPermissionsStatus;
 
 static ConfirmPermissionsStatus
-confirm_sharing_permissions (GtkWidget *widget, const char *path, gboolean is_shared, gboolean guest_ok, gboolean is_writable)
+confirm_sharing_permissions (PropertyPage *page, gboolean is_shared, gboolean guest_ok, gboolean is_writable)
 {
   struct stat st;
   mode_t mode, new_mode, need_mask;
+  SharingDialogData *data = g_new0 (SharingDialogData, 1);
+  data->page = page;
 
   if (!is_shared)
     return CONFIRM_NO_MODIFICATIONS;
 
-  if (stat (path, &st) != 0)
+  if (stat (page->path, &st) != 0)
     return CONFIRM_NO_MODIFICATIONS; /* We'll just let "net usershare" give back an error if the file disappears */
 
   new_mode = mode = st.st_mode;
@@ -340,18 +501,10 @@ confirm_sharing_permissions (GtkWidget *widget, const char *path, gboolean is_sh
     {
       g_assert (mode != new_mode);
 
-      if (!message_confirm_missing_permissions (widget, path, need_mask))
-	return CONFIRM_CANCEL_OR_ERROR;
-
-      if (chmod (path, new_mode) != 0)
-	{
-	  error_when_changing_permissions (widget, path);
-	  return CONFIRM_CANCEL_OR_ERROR;
-	}
-
-      save_changed_permissions (path, need_mask);
-
-      return CONFIRM_MODIFIED;
+      data->need_mask = need_mask;
+      data->new_mode = new_mode;
+      message_confirm_missing_permissions (data);
+      return CONFIRM_PENDING_CONFIRMATION;
     }
   else
     {
@@ -360,7 +513,7 @@ confirm_sharing_permissions (GtkWidget *widget, const char *path, gboolean is_sh
     }
 
   g_assert_not_reached ();
-  return CONFIRM_CANCEL_OR_ERROR;
+  return 0;
 }
 
 static gboolean
@@ -369,54 +522,20 @@ property_page_commit (PropertyPage *page)
   gboolean is_shared;
   ShareInfo share_info;
   ConfirmPermissionsStatus status;
-  GError *error;
-  gboolean retval;
 
-  is_shared = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_folder));
+  is_shared = check_button_get_active (page->checkbutton_share_folder);
 
-  share_info.path = page->path;
-  share_info.share_name = (char *) gtk_entry_get_text (GTK_ENTRY (page->entry_share_name));
-  share_info.comment = (char *) gtk_entry_get_text (GTK_ENTRY (page->entry_share_comment));
-  share_info.is_writable = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_rw_ro));
-  share_info.guest_ok = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_guest_ok));
+  share_info = get_share_info (page);
 
   /* Do we need to unset the write permissions that we added in the past? */
   if (is_shared && page->was_writable && !share_info.is_writable)
     restore_write_permissions (page->path);
 
-  status = confirm_sharing_permissions (page->main, page->path, is_shared, share_info.guest_ok, share_info.is_writable);
-  if (status == CONFIRM_CANCEL_OR_ERROR)
-    return FALSE; /* the user didn't want us to change his folder's permissions */
+  status = confirm_sharing_permissions (page, is_shared, share_info.guest_ok, share_info.is_writable);
+  if (status == CONFIRM_PENDING_CONFIRMATION)
+    return FALSE; /* wait for the response from dialog to see if the user wants to change permissions */
 
-  error = NULL;
-  retval = shares_modify_share (share_info.path, is_shared ? &share_info : NULL, &error);
-
-  if (!retval)
-    {
-      property_page_set_error (page, error->message);
-      g_error_free (error);
-
-      /* Since the operation failed, we restore things to the way they were */
-      if (status == CONFIRM_MODIFIED)
-	restore_saved_permissions (page->path);
-    }
-  else
-    {
-      property_page_validate_fields (page);
-      nautilus_file_info_invalidate_extension_info (page->fileinfo);
-    }
-
-  if (!is_shared)
-    restore_saved_permissions (page->path);
-
-  /* update initially shared state, so that we may undo later on */
-  if (retval)
-    {
-      page->was_initially_shared = is_shared;
-      page->is_dirty = FALSE;
-    }
-
-  return retval;
+  return property_page_apply_changes(page, FALSE);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -441,31 +560,37 @@ get_fullpath_from_fileinfo(NautilusFileInfo *fileinfo)
 static void
 property_page_set_warning (PropertyPage *page)
 {
+#ifdef USE_NAUTILUS_3
   GdkColor  colorYellow;
-
+#endif
   gtk_label_set_text (GTK_LABEL (page->label_status), _("Share name is too long"));
-
+#ifdef USE_NAUTILUS_3
   gdk_color_parse ("#ECDF62", &colorYellow);
   gtk_widget_modify_base (page->entry_share_name, GTK_STATE_NORMAL, &colorYellow);
+#endif
 }
 
 
 static void
 property_page_set_error (PropertyPage *page, const char *message)
 {
+#ifdef USE_NAUTILUS_3
   GdkColor colorRed;
-
+#endif
   gtk_label_set_text (GTK_LABEL (page->label_status), message);
-
+#ifdef USE_NAUTILUS_3
   gdk_color_parse ("#C1665A", &colorRed);
   gtk_widget_modify_base (page->entry_share_name, GTK_STATE_NORMAL, &colorRed);
+#endif
 }
 
 static void
 property_page_set_normal (PropertyPage *page)
 {
   gtk_label_set_text (GTK_LABEL (page->label_status), "");
+#ifdef USE_NAUTILUS_3
   gtk_widget_modify_base (page->entry_share_name, GTK_STATE_NORMAL, NULL);
+#endif
 }
 
 static gboolean
@@ -473,7 +598,7 @@ property_page_share_name_is_valid (PropertyPage *page)
 {
   const char *newname;
 
-  newname = gtk_entry_get_text (GTK_ENTRY (page->entry_share_name));
+  newname = entry_get_text (GTK_ENTRY (page->entry_share_name));
 
   if (strlen (newname) == 0)
     {
@@ -517,8 +642,8 @@ property_page_set_controls_sensitivity (PropertyPage *page,
 {
   gtk_widget_set_sensitive (page->entry_share_name, sensitive);
   gtk_widget_set_sensitive (page->entry_share_comment, sensitive);
-  gtk_widget_set_sensitive (page->hbox_share_comment, sensitive);
-  gtk_widget_set_sensitive (page->hbox_share_name, sensitive);
+  gtk_widget_set_sensitive (page->label_share_comment, sensitive);
+  gtk_widget_set_sensitive (page->label_share_name, sensitive);
   gtk_widget_set_sensitive (page->checkbutton_share_rw_ro, sensitive);
 
   if (sensitive)
@@ -537,7 +662,7 @@ property_page_check_sensitivity (PropertyPage *page)
   gboolean enabled;
   gboolean apply_is_sensitive;
 
-  enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_folder));
+  enabled = check_button_get_active (page->checkbutton_share_folder);
   property_page_set_controls_sensitivity (page, enabled);
 
   if (enabled)
@@ -582,7 +707,7 @@ modify_share_comment_text_entry  (GtkEditable *editable,
 
 /*--------------------------------------------------------------------------*/
 static void
-on_checkbutton_share_folder_toggled    (GtkToggleButton *togglebutton,
+on_checkbutton_share_folder_toggled    (GtkWidget       *togglebutton,
                                         gpointer         user_data)
 {
   PropertyPage *page;
@@ -593,8 +718,8 @@ on_checkbutton_share_folder_toggled    (GtkToggleButton *togglebutton,
 }
 
 static void
-on_checkbutton_rw_ro_toggled    (GtkToggleButton *togglebutton,
-				 gpointer         user_data)
+on_checkbutton_rw_ro_toggled    (GtkWidget *togglebutton,
+				                         gpointer   user_data)
 {
   PropertyPage *page;
 
@@ -606,8 +731,8 @@ on_checkbutton_rw_ro_toggled    (GtkToggleButton *togglebutton,
 }
 
 static void
-on_checkbutton_guest_ok_toggled    (GtkToggleButton *togglebutton,
-				    gpointer         user_data)
+on_checkbutton_guest_ok_toggled    (GtkWidget *togglebutton,
+				                            gpointer   user_data)
 {
   PropertyPage *page;
 
@@ -643,7 +768,7 @@ button_apply_clicked_cb (GtkButton *button,
   if (property_page_commit (page))
     {
       if (page->standalone_window)
-	gtk_widget_destroy (page->standalone_window);
+	      window_destroy (page->standalone_window);
       else
         property_page_check_sensitivity (page);
     }
@@ -687,8 +812,9 @@ create_property_page (NautilusFileInfo *fileinfo)
 
   page->xml = gtk_builder_new ();
   gtk_builder_set_translation_domain (page->xml, "nautilus-share");
-  g_assert (gtk_builder_add_from_file (page->xml,
-              INTERFACES_DIR"/share-dialog.ui", &error));
+  gtk_builder_add_from_file (page->xml, INTERFACE_FILE, &error);
+  if (error != NULL)
+    g_error ("%s", error->message);
 
   page->main = GTK_WIDGET (gtk_builder_get_object (page->xml, "vbox1"));
   g_assert (page->main != NULL);
@@ -699,24 +825,24 @@ create_property_page (NautilusFileInfo *fileinfo)
 			  free_property_page_cb);
 
   page->checkbutton_share_folder = GTK_WIDGET (gtk_builder_get_object (page->xml,"checkbutton_share_folder"));
-  page->hbox_share_comment = GTK_WIDGET (gtk_builder_get_object (page->xml,"hbox_share_comment"));
-  page->hbox_share_name = GTK_WIDGET (gtk_builder_get_object (page->xml,"hbox_share_name"));
   page->checkbutton_share_rw_ro = GTK_WIDGET (gtk_builder_get_object (page->xml,"checkbutton_share_rw_ro"));
   page->checkbutton_share_guest_ok = GTK_WIDGET (gtk_builder_get_object (page->xml,"checkbutton_share_guest_ok"));
   page->entry_share_name = GTK_WIDGET (gtk_builder_get_object (page->xml,"entry_share_name"));
   page->entry_share_comment = GTK_WIDGET (gtk_builder_get_object (page->xml,"entry_share_comment"));
+  page->label_share_comment = GTK_WIDGET (gtk_builder_get_object (page->xml,"label_share_comment"));
+  page->label_share_name = GTK_WIDGET (gtk_builder_get_object (page->xml,"label_share_name"));
   page->label_status = GTK_WIDGET (gtk_builder_get_object (page->xml,"label_status"));
   page->button_cancel = GTK_WIDGET (gtk_builder_get_object (page->xml,"button_cancel"));
   page->button_apply = GTK_WIDGET (gtk_builder_get_object (page->xml,"button_apply"));
 
   /* Sanity check so that we don't screw up the Glade file */
   g_assert (page->checkbutton_share_folder != NULL
-	    && page->hbox_share_comment != NULL
-	    && page->hbox_share_name != NULL
 	    && page->checkbutton_share_rw_ro != NULL
 	    && page->checkbutton_share_guest_ok != NULL
 	    && page->entry_share_name != NULL
 	    && page->entry_share_comment != NULL
+            && page->label_share_comment != NULL
+            && page->label_share_name != NULL
 	    && page->label_status != NULL
 	    && page->button_cancel != NULL
 	    && page->button_apply != NULL);
@@ -740,7 +866,7 @@ create_property_page (NautilusFileInfo *fileinfo)
       free_share_name = TRUE;
     }
 
-  gtk_entry_set_text (GTK_ENTRY (page->entry_share_name), share_name);
+  entry_set_text (GTK_ENTRY (page->entry_share_name), share_name);
 
   if (free_share_name)
     g_free (share_name);
@@ -752,33 +878,33 @@ create_property_page (NautilusFileInfo *fileinfo)
   else
     comment = share_info->comment;
 
-  gtk_entry_set_text (GTK_ENTRY (page->entry_share_comment), comment);
+  entry_set_text (GTK_ENTRY (page->entry_share_comment), comment);
 
   /* Share toggle */
 
   if (share_info)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_folder), TRUE);
+    check_button_set_active (page->checkbutton_share_folder, TRUE);
   else
     {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_folder), FALSE);
+      check_button_set_active (page->checkbutton_share_folder, FALSE);
     }
 
   /* Share name */
 
-  if (g_utf8_strlen(gtk_entry_get_text (GTK_ENTRY (page->entry_share_name)), -1) > 12)
+  if (g_utf8_strlen(entry_get_text (GTK_ENTRY (page->entry_share_name)), -1) > 12)
     property_page_set_warning (page);
 
   /* Permissions */
   if (share_info != NULL && share_info->is_writable)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_rw_ro), TRUE);
+    check_button_set_active (page->checkbutton_share_rw_ro, TRUE);
   else
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_rw_ro), FALSE);
+    check_button_set_active (page->checkbutton_share_rw_ro, FALSE);
 
   /* Guest access */
   if (share_info != NULL && share_info->guest_ok)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_guest_ok), TRUE);
+    check_button_set_active (page->checkbutton_share_guest_ok, TRUE);
   else
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->checkbutton_share_guest_ok), FALSE);
+    check_button_set_active (page->checkbutton_share_guest_ok, FALSE);
 
   /* Apply button */
 
@@ -789,7 +915,13 @@ create_property_page (NautilusFileInfo *fileinfo)
 
   gtk_button_set_label (GTK_BUTTON (page->button_apply), apply_button_label);
   gtk_button_set_use_underline (GTK_BUTTON (page->button_apply), TRUE);
-  gtk_button_set_image (GTK_BUTTON (page->button_apply), gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_BUTTON));
+#ifdef USE_NAUTILUS_3
+  gtk_button_set_image (GTK_BUTTON (page->button_apply),
+			gtk_image_new_from_icon_name ("document-save" ,GTK_ICON_SIZE_BUTTON));
+#endif
+#ifdef USE_NAUTILUS_4
+  gtk_button_set_icon_name (GTK_BUTTON (page->button_apply), "document-save");
+#endif
 
   gtk_widget_set_sensitive (page->button_apply, FALSE);
 
@@ -998,7 +1130,7 @@ nautilus_share_cancel_update (NautilusInfoProvider *provider,
 }
 
 static void 
-nautilus_share_info_provider_iface_init (NautilusInfoProviderIface *iface)
+nautilus_share_info_provider_iface_init (NautilusInfoProviderInterface *iface)
 {
   iface->update_file_info = nautilus_share_update_file_info;
   iface->cancel_update = nautilus_share_cancel_update;
@@ -1016,6 +1148,7 @@ nautilus_share_info_provider_iface_init (NautilusInfoProviderIface *iface)
  * The function should return a GList of allocated NautilusPropertyPage
  * items.
  */
+#ifdef USE_NAUTILUS_3
 static GList *
 nautilus_share_get_property_pages (NautilusPropertyPageProvider *provider,
 				   GList *files)
@@ -1060,7 +1193,7 @@ nautilus_share_property_page_provider_iface_init (NautilusPropertyPageProviderIf
 {
   iface->get_pages = nautilus_share_get_property_pages;
 }
-
+#endif
 /*--------------------------------------------------------------------------*/
 static void
 nautilus_share_instance_init (NautilusShare *share)
@@ -1092,7 +1225,7 @@ button_cancel_clicked_cb (GtkButton *button, gpointer data)
   GtkWidget *window;
 
   window = GTK_WIDGET (data);
-  gtk_widget_destroy (window);
+  window_destroy (window);
 }
 
 static void
@@ -1106,20 +1239,31 @@ share_this_folder_callback (NautilusMenuItem *item,
   fileinfo = NAUTILUS_FILE_INFO (user_data);
   g_assert (fileinfo != NULL);
 
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  window = gtk_window_new (
+#ifdef USE_NAUTILUS_3
+	                   GTK_WINDOW_TOPLEVEL
+#endif
+			   );
   gtk_window_set_title (GTK_WINDOW (window), _("Folder Sharing"));
   page = create_property_page (fileinfo);
   page->standalone_window = window;
   g_signal_connect (page->button_cancel, "clicked",
 		    G_CALLBACK (button_cancel_clicked_cb), window);
 
+#ifdef USE_NAUTILUS_3
   gtk_container_add (GTK_CONTAINER (window), page->main);
+#endif
+#ifdef USE_NAUTILUS_4
+  gtk_window_set_child (GTK_WINDOW (window), GTK_WIDGET (page->main));
+#endif
   gtk_widget_show (window);
 }
 
 static GList *
 nautilus_share_get_file_items (NautilusMenuProvider *provider,
+#ifdef USE_NAUTILUS_3
 			     GtkWidget *window,
+#endif
 			     GList *files)
 {
   GList *items;
@@ -1165,7 +1309,7 @@ nautilus_share_get_file_items (NautilusMenuProvider *provider,
 
 /*--------------------------------------------------------------------------*/
 static void 
-nautilus_share_menu_provider_iface_init (NautilusMenuProviderIface *iface)
+nautilus_share_menu_provider_iface_init (NautilusMenuProviderInterface *iface)
 {
 	iface->get_file_items = nautilus_share_get_file_items;
 }
@@ -1204,8 +1348,7 @@ nautilus_share_register_type (GTypeModule *module)
 					    G_TYPE_OBJECT,
 					    "NautilusShare",
 					    &info, 0);
-
-  /* onglet share proprit */
+#ifdef USE_NAUTILUS_3
   static const GInterfaceInfo property_page_provider_iface_info = {
     (GInterfaceInitFunc) nautilus_share_property_page_provider_iface_init,
     NULL,
@@ -1216,9 +1359,8 @@ nautilus_share_register_type (GTypeModule *module)
 			       share_type,
 			       NAUTILUS_TYPE_PROPERTY_PAGE_PROVIDER,
 			       &property_page_provider_iface_info);
+#endif
 
-
-  /* premier page proprit ? */
   static const GInterfaceInfo info_provider_iface_info = {
     (GInterfaceInitFunc) nautilus_share_info_provider_iface_init,
     NULL,
